import sys, os
import numpy as np

import BioNetInput

#
# Data
#

# Constants
numNodes = 10000
radius = 1.0
kbt = 4.11
crossk = kbt / np.power(radius / 10.0, 2)
crossl = 0.15
crossE = 576
vp = (4.0 / 3.0) * np.pi * np.power(radius, 3.0)
mu = 1

# Variables
vfs = [0.005, 0.01, 0.05, 0.07, 0.1]
ns = [4,6,8,14]
rates = [1000]

# Site stuff
sPos = []
sPos.append([np.array([0.816497,0.0,-0.57735]), np.array([-0.816497,0.0,-0.57735]), np.array([0.0,0.816497,0.57735]), np.array([0.0,-0.816497,0.57735])])
sPos.append([np.array([1.0,0.0,0.0]), np.array([-1.0,0.0,0.0]), np.array([0.0,1.0,0.0]), np.array([0.0,-1.0,0.0]), np.array([0.0,0.0,1.0]), np.array([0.0,0.0,-1.0])])
sPos.append([np.array([0.57735, 0.57735, 0.57735]), np.array([-0.57735, 0.57735, 0.57735]), np.array([0.57735, -0.57735, 0.57735]), np.array([0.57735, 0.57735, -0.57735]), np.array([-0.57735, -0.57735, 0.57735]), np.array([-0.57735, 0.57735, -0.57735]), np.array([0.57735, -0.57735, -0.57735]), np.array([-0.57735, -0.57735, -0.57735])])

sPos.append([])
sPos[-1].extend(sPos[-3])
sPos[-1].extend(sPos[-2])

#
# Build the scripts
#

inp = BioNetInput.BioNetInput()

# Protein
prot = BioNetInput.BioNetInputProtein()
prot.name = "ProteinA"
prot.type = "sphere"
prot.density = 1e3
prot.viscosity = 1e-3

inp.protein.append(prot)
inp.numProteins += 1

# Bond
bon = BioNetInput.BioNetInputBond()
bon.name = "CrosslinkA"
bon.type = "spring"
bon.k = crossk
bon.l = crossl
bon.E = crossE
bon.cutoff = 2 * bon.l

inp.bond.append(bon)
inp.numBonds += 1

# Forces (Steric)
force = BioNetInput.BioNetInputForce()
force.name = "steric"
force.attributes["k"] = 1000	# 1kT for 0.3 overlap 

# Parameters
params = BioNetInput.BioNetInputParameters()

# Simulation parameters
drag = 6 * np.pi * mu * radius
tau = drag / crossk

params.dt = tau / 10.0

# Network parameters
params.viscosity = 1e-3
params.temperature = 298
params.kbt = kbt

inp.params = params

# Loop over variables
for rate in rates:

	# Protein addition
	inp.protein[-1].rate = rate

	sIndex = 0
	for s in ns:

		# Protein addition
		inp.protein[-1].maxNumConnections = s
		inp.protein[-1].numSites = s
		inp.protein[-1].site = []

		# Sites
		for i in range(s):
			sit = BioNetInput.BioNetInputSite()
			sit.name = "Site" + chr(65 + i)
			sit.type = "point"
			sit.position = sPos[sIndex][i]
			sit.rate = rate

			inp.protein[-1].site.append(sit)

		for vf in vfs:
			Lb = np.power(numNodes * vp / vf, 1.0/3.0)

			# Box
			box = BioNetInput.BioNetInputSimulationBox()
			box.BC = "HBC"
			box.dimension = np.array([Lb, Lb, Lb])
			box.numVoxels = np.array([25,25,25])

			inp.box = box

			# Nodes!
			inp.numNodes = numNodes
			inp.node = []
			for i in range(numNodes):
				node = BioNetInput.BioNetInputNode()
				node.protein = inp.protein[-1]
				node.radius = radius

				# Randomise these things
				for j in range(3):

					# In the box
					node.pos[j] = np.random.rand() * (Lb - 2 * radius) + radius
					node.orientation[j] = np.random.rand() * 2 * np.pi

				inp.node.append(node)

			# Finalise by calculating the appropriate relaxation time
			rotDrag = 8 * np.pi * mu * np.power(radius,3)
			tBoxDif = 0.5 *np.power(Lb / 2.0,2) * drag / kbt
			tRotDif = 0.5 *np.power(2 * np.pi,2) * rotDrag / kbt

			# Because we're just relaxing here
			inp.params.simTime = np.max([tBoxDif, tRotDif])
			inp.params.timePerFrame = int(inp.params.simTime / 10.0)
			if inp.params.timePerFrame < inp.params.dt:
				inp.params.timePerFrame = inp.params.dt

			# Write to a file
			oFname = "../input/N%dSites%02dRadius%05.2fRate%5.2fvf%0.4fRelax.xml" % (numNodes, s, radius, rate, vf)
			inp.writeToFile(oFname)

		sIndex += 1
